@@ -1,4 +1,4 @@
-/* 02sep17abu
+/* 14jun18abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -1024,11 +1024,15 @@ static void comment(void) {
       }
    }
    else {
+      int n = 0;
+
       for (;;) {  // #{block-comment}# from Kriangkrai Soatthiyanont
          Env.get();
          if (Chr < 0)
             return;
-         if (Chr == '}' && (Env.get(), Chr == '#'))
+         if (Chr == '#'  &&  (Env.get(), Chr == '{'))
+            ++n;
+         else if (Chr == '}'  &&  (Env.get(), Chr == '#')  &&  --n < 0)
             break;
       }
       Env.get();
@@ -1478,7 +1482,7 @@ long waitFd(any ex, int fd, long ms) {
          tv.tv_usec = t % 1000 * 1000;
 #ifndef __linux__
          gettimeofday(&tt,NULL);
-         t = tt.tv_sec*1000 + tt.tv_usec/1000;
+         t = tt.tv_sec*1000 + (tt.tv_usec + 500) / 1000;
 #endif
       }
       while (select(m+1, &rdSet, &wrSet, NULL, tp) < 0) {
@@ -1491,10 +1495,10 @@ long waitFd(any ex, int fd, long ms) {
       }
       if (tp) {
 #ifdef __linux__
-         t -= tv.tv_sec*1000 + tv.tv_usec/1000;
+         t -= tv.tv_sec*1000 + (tv.tv_usec + 500) / 1000;
 #else
          gettimeofday(&tt,NULL);
-         t = tt.tv_sec*1000 + tt.tv_usec/1000 - t;
+         t = tt.tv_sec*1000 + (tt.tv_usec + 500) / 1000 - t;
 #endif
          if (ms > 0  &&  (ms -= t) < 0)
             ms = 0;
@@ -2270,9 +2274,9 @@ any doEcho(any ex) {
    long cnt;
 
    x = cdr(ex),  y = EVAL(car(x));
-   if (!Chr)
-      Env.get();
    if (isNil(y) && !isCell(cdr(x))) {
+      if (!Chr)
+         Env.get();
       while (Chr >= 0)
          Env.put(Chr),  Env.get();
       return T;
@@ -2291,6 +2295,8 @@ any doEcho(any ex) {
          y = evSym(x = cdr(x));
       }
       m = -1;
+      if (!Chr)
+         Env.get();
       while (Chr >= 0) {
          if ((om = m) >= 0)
             op = p[m];
@@ -2340,18 +2346,18 @@ any doEcho(any ex) {
       return x;
    }
    if (isCell(x = cdr(x))) {
-      for (cnt = xCnt(ex,y), y = EVAL(car(x)); --cnt >= 0; Env.get())
+      for (cnt = xCnt(ex,y), y = EVAL(car(x)); --cnt >= 0;) {
+         Env.get();
          if (Chr < 0)
             return Nil;
+      }
    }
    if ((cnt = xCnt(ex,y)) > 0) {
-      for (;;) {
+      while (--cnt >= 0) {
+         Env.get();
          if (Chr < 0)
             return Nil;
          Env.put(Chr);
-         if (!--cnt)
-            break;
-         Env.get();
       }
    }
    Chr = 0;
