@@ -1,4 +1,4 @@
-/* 17nov17abu
+/* 01apr18abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -19,6 +19,7 @@
 #include <openssl/pem.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
+#include <openssl/x509v3.h>
 
 typedef enum {NO,YES} bool;
 
@@ -148,12 +149,12 @@ static void iSignal(int n, void (*foo)(int)) {
 }
 
 // ssl host port [url key file]
-// ssl host port url key file dir sec [min]
+// ssl host port url key file dir sec [min] [dir ..]
 int main(int ac, char *av[]) {
    bool dbg;
    SSL_CTX *ctx;
    SSL *ssl;
-   int n, sec, lim, getLen, lenLen, fd, sd;
+   int i, n, sec, lim, getLen, lenLen, fd, sd;
    DIR *dp;
    struct dirent *p;
    struct stat st;
@@ -161,8 +162,8 @@ int main(int ac, char *av[]) {
 
    if (dbg = strcmp(av[ac-1], "+") == 0)
       --ac;
-   if (!(ac >= 3 && ac <= 6  ||  ac >= 8 && ac <= 9))
-      giveup("host port [url key file] | host port url key file dir sec [min]");
+   if (!(ac >= 3 && ac <= 6  ||  ac >= 8))
+      giveup("host port [url key file] | host port url key file dir sec [min] [dir ..]");
    if (*av[2] == '-')
       ++av[2],  Safe = YES;
    if (ac <= 3  ||  *av[3] == '\0')
@@ -183,6 +184,12 @@ int main(int ac, char *av[]) {
       SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_ALL | SSL_OP_NO_COMPRESSION );
    SSL_CTX_set_cipher_list(ctx, Ciphers);
    ssl = SSL_new(ctx);
+   if (!Safe) {
+      X509_VERIFY_PARAM *par = SSL_get0_param(ssl);
+      X509_VERIFY_PARAM_set_hostflags(par, X509_CHECK_FLAG_NO_PARTIAL_WILDCARDS);
+      X509_VERIFY_PARAM_set1_host(par, av[1], 0);
+      SSL_set_verify(ssl, SSL_VERIFY_PEER, NULL);
+   }
 
    signal(SIGCHLD,SIG_IGN);  /* Prevent zombies */
    signal(SIGPIPE, SIG_IGN);
@@ -225,7 +232,7 @@ int main(int ac, char *av[]) {
    iSignal(SIGINT, doSigTerm);
    iSignal(SIGTERM, doSigTerm);
    lim = 0;
-   if (ac == 9) {
+   if (ac >= 9 && *av[8]) {
       if ((lim = atoi(av[8])) == 0) {
          printf("%d\n", getpid());
          fflush(stdout);
@@ -253,6 +260,32 @@ int main(int ac, char *av[]) {
             if (ftruncate(fd,0) < 0)
                giveup("Can't truncate");
             close(fd);
+            for (nm[0] = '\0', i = 9;  i < ac;  ++i) {
+               if (dp = opendir(av[i])) {
+                  int max = -1;
+                  char *q;
+
+                  while (p = readdir(dp))
+                     if ((n = (int)strtol(p->d_name, &q, 10)) > max  &&  !*q)
+                        max = n;
+                  if (max >= 0) {
+                     if (nm[0]) {
+                        snprintf(buf, sizeof(buf), "%s/%d", av[i], max + 1);
+                        link(nm, buf);
+                     }
+                     else {
+                        snprintf(nm, sizeof(nm), "%s/%d", av[i], max + 1);
+                        if ((fd = open(nm, O_CREAT|O_EXCL|O_WRONLY, 0666)) < 0)
+                           nm[0] = '\0';
+                        else {
+                           write(fd, Data, Size);
+                           close(fd);
+                        }
+                     }
+                  }
+                  closedir(dp);
+               }
+            }
             for (;;) {
                if ((sd = sslConnect(ssl, av[1], av[2])) >= 0) {
                   if (SSL_write(ssl, get, getLen) == getLen  &&
